package cn.com.qmth.examcloud.core.questions.api.controller;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseByOrgReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseByOrgResp;
import cn.com.qmth.examcloud.core.questions.service.CopyDataService;
import cn.com.qmth.examcloud.core.questions.service.bean.CopyDataCourseInfo;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.ServletUtil;

@RestController
@RequestMapping("${api_cqb}/copy")
public class CopyDataController extends ControllerSupport {
	private static int cacheLockTimeout = 60 * 10;
	private static String copyQuestionDataLock = "$_COPY_QUESTION_DATA_LOCK";
	@Autowired
	private CopyDataService copyDataService;
	@Autowired
	private RedisClient redisClient;
	@Autowired
	private CourseCloudService courseCloudService;

	@PostMapping(value = "data")
	public void copyData(HttpServletResponse response,
			@RequestPart @NotNull(message = "上传文件不能为空！") MultipartFile dataFile, @RequestParam Long fromRootOrgId,
			@RequestParam Long toRootOrgId) {
		User user = getAccessUser();
		if (!user.getRootOrgId().equals(toRootOrgId)) {
			ServletUtil.returnJson("请求失败，没有权限", response);
			return;
		}
		Boolean lock = redisClient.setIfAbsent(copyQuestionDataLock, copyQuestionDataLock, cacheLockTimeout);
		if (!lock) {
			ServletUtil.returnJson("请求失败，正在处理数据", response);
			return;
		}
		try {
			LinkedHashSet<String> cs = readSubject(dataFile);
			if (CollectionUtils.isEmpty(cs)) {
				throw new StatusException("请填写要处理的课程");
			}
			Map<String, Long> coursesMap = new HashMap<>();
			GetCourseByOrgReq courseReq = new GetCourseByOrgReq();
			courseReq.setRootOrgId(toRootOrgId);
			GetCourseByOrgResp res = courseCloudService.getCourseByOrg(courseReq);
			if (CollectionUtils.isEmpty(res.getCourseList())) {
				throw new StatusException("请先导入课程");
			}

			for (CourseBean c : res.getCourseList()) {
				coursesMap.put(c.getCode(), c.getId());
			}
			List<CopyDataCourseInfo> courses = new ArrayList<>();
			for (String c : cs) {
				if (coursesMap.get(c) == null) {
					throw new StatusException("未找到课程信息:" + c);
				} else {
					courses.add(new CopyDataCourseInfo(coursesMap.get(c), c));
				}
			}
			copyDataService.copyData(user, fromRootOrgId, toRootOrgId, courses);
			ServletUtil.returnJson("请求成功", response);
		} catch (StatusException e) {
			ServletUtil.returnJson("请求失败" + e.getDesc(), response);
			redisClient.delete(copyQuestionDataLock);
		} catch (Exception e) {
			ServletUtil.returnJson("请求失败" + e.getMessage(), response);
			redisClient.delete(copyQuestionDataLock);
			throw e;
		}

	}

	private LinkedHashSet<String> readSubject(MultipartFile dataFile) {
		LinkedHashSet<String> list = new LinkedHashSet<>();
		XSSFWorkbook wb = null;
		InputStream in = null;
		try {
			in = dataFile.getInputStream();
			wb = new XSSFWorkbook(in);
			XSSFSheet sheet = wb.getSheetAt(0);
			int rows = sheet.getLastRowNum();
			for (int i = 1; i <= rows; i++) {
				XSSFRow row = sheet.getRow(i);
				String code;
				try {
					code = row.getCell(0).getStringCellValue().trim();
				} catch (Exception e) {
					code = (row.getCell(0).getNumericCellValue() + "").trim();
				}
				list.add(code);
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
				}
			}
			if (wb != null) {
				try {
					wb.close();
				} catch (IOException e) {
				}
			}
		}
		return list;
	}
	// 试题去重后复制
//	@GetMapping("noreduplicate")
//	public void copyNoReduplicate(HttpServletResponse response,@RequestParam String batch) {
//		Boolean lock = redisClient.setIfAbsent(withoutReduplicateLock, withoutReduplicateLock, cacheLockTimeout);
//		if (!lock) {
//			ServletUtil.returnJson("请求失败，正在处理数据", response);
//			return;
//		}
//		copyDataService.copyNoReduplicate(getAccessUser(), batch);
//		ServletUtil.returnJson("请求成功", response);
//	}

	@GetMapping("status")
	public void copyNoReduplicateStatus(HttpServletResponse response) {
		String lock = redisClient.get(copyQuestionDataLock, String.class);
		if (lock != null) {
			ServletUtil.returnJson("正在处理数据", response);
		} else {
			ServletUtil.returnJson("处理数据结束", response);
		}
	}

}
